// Copyright 2012 (www.planetarywebdesign.com). All Rights Reserved.
// Licensed under the MIT License.
// you may not use this file except in compliance with the License.
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
* @fileOverview Highlights the syntax in JavaScript code wrapped in <pre></pre> tags.
*/

/** 
* @class
* A class that consists of the regex matches and the html with the regex matches
* stripped from it.
*/
var stringMatcher = function (html) {
    this.html = html;
    this.matches = [];

    /**
    * Wrap all the stripped matches
    */
    this.wrap = function () {
        settings = {
            className: "COMM"
        };

        if (this.matches.length == 0)
            return;



        var totalExtraLength = 0;
        for (var i = 0; i < this.matches.length; i++) {
            var wrapStart = "<span class=\"" + this.matches[i].settings.className + "\">";
            var wrapEnd = "</span>";
            var middle = wrapStart + this.matches[i][0] + wrapEnd;
            this.html = this.html.substr(0, this.matches[i].index + totalExtraLength)
				+ middle
				+ this.html.substr(this.matches[i].index + totalExtraLength);
            totalExtraLength += middle.length;
        }

    }

    /**
    * Strip out and take a record of the matches from a regex.
    * @param regex The regex to match the sections to strip out.
    * @param settings.className The class name to give the span that
    * will wrap the stripped out section when 'wrap' is called.
    * @remarks Making subsequent calls to this function will mean the
    * regex will never match the pervious stripped out html, useful for
    * stripping out sections you don't want to match in order.
    */
    this.strip = function (regex, settings) {

        settings = settings || {};
        settings = {
            className: settings.className || ""
        };

        var matches = [];
        while (m = regex.exec(this.html)) {
            // Give the regex a copy of these settings.
            m.settings = settings;
            matches.push(m);
        };

        // strip the matched html
        var totalExtraLength = 0;
        for (var i = 0; i < matches.length; i++) {
            var middle = "";
            this.html = this.html.substr(0, matches[i].index + totalExtraLength)
				+ middle
				+ this.html.substr(matches[i].index + matches[i][0].length + totalExtraLength);
            matches[i].index += totalExtraLength;
            totalExtraLength -= matches[i][0].length;

            // Add matches to internal matches
            this.addMatch(matches[i]);
        }

    };

    /**
    * Add a match to the string matcher, adds it to the correct
    * index in the array of matches using the regex match index property.
    * @param match The regex match to add.
    */
    this.addMatch = function (match) {
        var index = match.index;
        var addedOnce = false;
        for (var i = 0; i < this.matches.length; i++) {
            var curIndex = this.matches[i].index;
            if (curIndex > index) {
                // Alter the index for any matches that come afterwards
                this.matches[i].index -= match[0].length;
            }
            if ((!addedOnce) && curIndex > index) {
                this.matches.splice(i, 0, match);
                addedOnce = true;
                i++;
            }
        }
        if (addedOnce == undefined || addedOnce === false)
            this.matches.push(match);
    };

}





// Add some compatibility for functions we want to use.
String.prototype.lines = function () { return this.split(/\n/g); }
String.prototype.lineCount = function () { return this.lines().length; }
if (!String.prototype.trim) {
    String.prototype.trim = function () { return this.replace(/^\s\s*/, '').replace(/\s\s*$/, ''); };
}
if (!Array.prototype.indexOf) {
    Array.prototype.indexOf = function (searchElement /*, fromIndex */) {
        "use strict";
        if (this == null) {
            throw new TypeError();
        }
        var t = Object(this);
        var len = t.length >>> 0;
        if (len === 0) {
            return -1;
        }
        var n = 0;
        if (arguments.length > 1) {
            n = Number(arguments[1]);
            if (n != n) { // shortcut for verifying if it's NaN
                n = 0;
            } else if (n != 0 && n != Infinity && n != -Infinity) {
                n = (n > 0 || -1) * Math.floor(Math.abs(n));
            }
        }
        if (n >= len) {
            return -1;
        }
        var k = n >= 0 ? n : Math.max(len - Math.abs(n), 0);
        for (; k < len; k++) {
            if (k in t && t[k] === searchElement) {
                return k;
            }
        }
        return -1;
    }
}

var SyntaxHighlighter = {};
(function () {
    var self = this;

    this.settings = {
        elementsTagNameToHighlight: 'pre'
    };

    this.ie = (function () {
        var undef,
            v = 3,
            div = document.createElement('div'),
            all = div.getElementsByTagName('i');
        while (
            div.innerHTML = '<!--[if gt IE ' + (++v) + ']><i></i><![endif]-->',
            all[0]
        );
        return v > 4 ? v : undef;
    } ());

    /**
    * Styles needed for the syntax highlighter
    */
    this.styles = ".KEYW {color: #00f;}"
		+ ".COMM {color: #bbb; font-style: italic;}"
		+ ".NUMB {color: #ab0034;}" /* Not implemented yet */
		+ ".STRN {color: #ab0034;}" /* Not implemented yet */
		+ ".REGX {color: #ab0034;}" /* Not implemented yet */

		+ ".line-numbers, .line-number {"
			+ "float: left;"
			+ "-webkit-touch-callout: none;"
			+ "-webkit-user-select: none;"
			+ "-khtml-user-select: none;"
			+ "-moz-user-select: none;"
			+ "-ms-user-select: none;"
			+ "user-select: none;"
			+ "-o-user-select: none;"
			+ "user-select: none;"

			+ "border-right:  1px dotted #666;"
			+ "padding-right: 2px;"
			+ "color: #2b91af;"
			+ "margin-right: 4px;"
		+ "}";



    /**
    * Get all the elements on the page to apply highlighting to.
    */
    this.getElementsToParse = function () {
        var elements = document.getElementsByTagName(this.settings.elementsTagNameToHighlight);
        return elements;
    };

    /**
    * Keep a record of elements that have previously
    * been parsed by the syntax highlighter
    */
    this.elementsParsed = [];

    /**
    * Highlight all the scripts on the page.
    */
    this.highlightAll = function () {
        this.applyStyles();

        var elements = this.getElementsToParse();
        for (var i = 0; i < elements.length; i++) {
            if (this.elementsParsed.indexOf(elements[i]) == -1) {
                this.parseElement(elements[i]);
                this.elementsParsed[this.elementsParsed.length] = elements[i];
            }
        };
    };

    /**
    * Add the styles to the head section of the document if they don't already exist.
    */
    this.applyStyles = function () {
        // Check if styles already exist
        if (document.getElementById('SyntaxHighlighterStyles') != null)
            return;

        // Not using document.write to apply the css styling as it does not seem to work with Blogger
        var head = document.getElementsByTagName('head')[0];
        style = document.createElement('style');
        style.id = "SyntaxHighlighterStyles";
        style.type = 'text/css';
        if (style.styleSheet) {
            style.styleSheet.cssText = SyntaxHighlighter.styles;
        } else {
            style.appendChild(document.createTextNode(SyntaxHighlighter.styles));
        }
        head.appendChild(style);
    };

    /**
    * Alias for highlight all
    */
    this.all = this.highlightAll;

    /**
    * Parse and highlight the provided element.
    */
    this.parseElement = function (element) {

        if (element.innerText) {
            var html = element.innerText;

        } else {
            var html = element.innerHTML;
        }
        html = html.trim();
        html = this.parseHtml(html);
        // Add line numbers

        //if (element.innerText) {
        //	element.innerText = lineNumbers + html;
        //} else {

        var wrap = document.createElement("span");
        element.parentNode.replaceChild(wrap, element);

        if (false && this.ie != null && this.ie < 9) {
            // Method not in use

            // lower versions of ie don't seem to line up properly with the line-heights
            // so using this method, the disadvantage is that the line numbers are still selectable
            html = this.addLineNumbers(html);
            wrap.innerHTML = "<pre style=\"line-height: 14px;\">" + html + "</pre>";
        } else {
            wrap.innerHTML = "<pre style=\"line-height: 14px;\">" + this.getLineNumbers(html) + html + "</pre>";
        }

        //element.innerHTML = lineNumbers + html;
        //}
    };

    /**
    * Get a span element with only line numbers equal to the total lines of the html.
    */
    this.getLineNumbers = function (html) {
        var lineTotal = html.lineCount();
        lineNumbers = "<span style=\"line-height: 14px;\" class=\"line-numbers\" unselectable=\"on\">";
        var length = lineTotal.toString().length;
        var i = 1;
        while (i <= lineTotal) {
            for (var j = 0; j < length - (i).toString().length; j++) {
                lineNumbers += "&nbsp;";
            }
            lineNumbers += i + '\r\n';
            i++;
        }
        lineNumbers += "</span>";

        return lineNumbers;
    };

    /**
    * Adds line numbers to the html
    */
    this.addLineNumbers = function (html) {
        var lines = html.lines();
        var lineTotal = html.lineCount();

        var length = lineTotal.toString().length;
        var newHtml = "";
        for (var i = 0; i < lines.length; i++) {
            var number = "";
            for (var j = 0; j < length - (i + 1).toString().length; j++) {
                number += "&nbsp;";
            }
            number += (i + 1);
            newHtml += "<span class=\"line-number\" unselectable=\"on\">" + number + "</span>" + lines[i] + '\r\n';
        }
        newHtml = newHtml.slice(0, -2);
        return newHtml;
    };


    /**
    * Parse html to add the extra html needed for syntax highlighting.
    */
    this.parseHtml = function (html) {
        var handler = new stringMatcher(html);
        regexs = {
            multiLineCComments: /\/\*[\s\S]*?\*\//gm,
            singleLineCComments: /\/\/.*$/gm,
            doubleQuotedString: /"([^\\"\n]|\\.)*"/g,
            singleQuotedString: /'([^\\'\n]|\\.)*'/g,
            url: /\w+:\/\/[\w-.\/?%&=:@;]*/g,
            reservedWords: /(break|case|catch|continue|debugger|default|delete|do|else|finally|for|function|if|in|instanceof|new|return|switch|this|throw|try|typeof|var|void|while|with){1}(\s|\{){1}/gi
        };

        handler.strip(regexs.multiLineCComments, { className: "COMM" });
        handler.strip(regexs.singleLineCComments, { className: "COMM" });
        handler.strip(regexs.doubleQuotedString, { className: "STRN" });
        handler.strip(regexs.singleQuotedString, { className: "STRN" });
        handler.strip(regexs.reservedWords, { className: "KEYW" });
        handler.wrap();

        return handler.html;
    };

    /**
    * Cross browser add event function
    */
    this.addEvent = function (event, target, method) {
        if (target.addEventListener) {
            target.addEventListener(event, method, false);
        } else if (target.attachEvent) {
            target.attachEvent("on" + event, method);
        }
    }

}).apply(SyntaxHighlighter);

SyntaxHighlighter.addEvent("load", window, function () {
    SyntaxHighlighter.highlightAll();
});